using ldap_cesi.DTOs.Inputs.Salarie;
using ldap_cesi.DTOs.Inputs.Service;
using ldap_cesi.Services.Interfaces;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;

namespace ldap_cesi.Controllers;
[ApiController]
[Route("api/salaries")]
public class SalarieController : ControllerBase
{
    private ISalarieService _salarieService;

    public SalarieController(ISalarieService salarieService)
    {
        _salarieService = salarieService;
    }
    
    /// <summary>
    /// Endpoint qui retourne tous les salariés.
    /// </summary>
    /// <returns>Une liste de salariés.</returns>
    [HttpGet]
    public async Task<IActionResult> GetAllSalaries([FromQuery] int pageNumber = 1, [FromQuery] int pageSize = 25)
    {
        var result = await _salarieService.GetAll(pageNumber, pageSize);
        return result.Success ? Ok(result) : BadRequest(result);
    }
    
    
    [HttpGet("search")]
    public async Task<IActionResult> SearchSalaries(
        [FromQuery] string searchTerm,
        [FromQuery] int pageNumber = 1,
        [FromQuery] int pageSize = 10)
    {
        if (string.IsNullOrWhiteSpace(searchTerm) || searchTerm.Length < 2)
        {
            return BadRequest("Votre recherche doit contenir au moins deux caractères.");
        }

        var result = await _salarieService.SearchWithRelations(
            searchTerm, pageNumber, pageSize, s => s.IdServiceNavigation, s => s.IdSiteNavigation);

        return result.Success ? Ok(result) : BadRequest(result);
    }

    /// <summary>
    /// Endpoint qui retourne le salarié correspondant à l'ID en paramètre.
    /// </summary>
    /// <param name="id">L'ID du salarié.</param>
    /// <returns>Le salarié correspondant à l'ID.</returns>
    [HttpGet("{id}")]
    public async Task<IActionResult> GetById(int id)
    {
        var result = await _salarieService.GetById(id);
        return result.Success ? Ok(result) : BadRequest(result);
    }
    
    /// <summary>
    /// Endpoint qui retourne le salarié correspondant à l'ID en paramètre. Avec le nom de service et de site auxquels il apaprtient.
    /// </summary>
    /// <param name="id">L'ID du salarié.</param>
    /// <returns>Le salarié correspondant à l'ID. Ainsi que son service et son site</returns>
    [HttpGet("/complet/{id}")]
    public async Task<IActionResult> GetSalarieCompletById(int id)
    {
        var result = await _salarieService.GetByIdWithRelations(id,s => s.IdServiceNavigation, s => s.IdSiteNavigation);
        return result.Success ? Ok(result) : BadRequest(result);
    }    
    
    /// <summary>
    /// Endpoint qui retourne tout les salariés et les relations qu'il détient
    /// </summary>
    /// <returns>Tous les salariés avec leurs relations</returns>
    [HttpGet("all")]
    public async Task<IActionResult> GetAllSariesWithRelations([FromQuery] int pageNumber = 1, [FromQuery] int pageSize = 25)
    {
        var result = await _salarieService.GetAllWithRelationsAsync(pageNumber, pageSize,s => s.IdServiceNavigation, s => s.IdSiteNavigation);
        return result.Success ? Ok(result) : BadRequest(result);
    }

    /// <summary>
    /// Endpoint qui crée un salarié.
    /// </summary>
    /// <param name="salarieInput">Les informations du salarié à créer.</param>
    /// <returns>Le salarié créé.</returns>
    [HttpPost]
    [Authorize(Roles = "admin")]
    public async Task<IActionResult> CreateSalarie([FromBody] SalarieCreateDto salarieInput)
    {
        var result = await _salarieService.Create(salarieInput);
        return result.Success ? Ok(result) : BadRequest(result);
    }

    /// <summary>
    /// Endpoint qui met à jour un salarié.
    /// </summary>
    /// <param name="salarieInput">Les informations du salarié à mettre à jour.</param>
    /// <returns>Le salarié mis à jour.</returns>
    [HttpPut]
    [Authorize(Roles = "admin")]
    public async Task<IActionResult> UpdateSalarie([FromBody] SalarieUpdateDto salarieInput)
    {
        var result = await _salarieService.Update(salarieInput);
        return result.Success ? Ok(result) : BadRequest(result);
    }

    /// <summary>
    /// Endpoint qui supprime un salarié.
    /// </summary>
    /// <param name="id">L'ID du salarié à supprimer.</param>
    /// <returns>Un message de confirmation de suppression.</returns>
    [HttpDelete("{id}")]
    [Authorize(Roles = "admin")]
    public async Task<IActionResult> DeleteSalarie(int id)
    {
        var result = await _salarieService.Delete(id);
        return result.Success ? Ok(result) : BadRequest(result);
    }
    
    /// <summary>
    /// Récupère les salariés appartenant à un site spécifique
    /// </summary>
    /// <param name="siteId">Identifiant du site</param>
    /// <returns>Liste des salariés du site</returns>
    [HttpGet("site/{siteId}")]
    public async Task<IActionResult> GetSalariesBySite(int siteId)
    {
        var result = await _salarieService.GetSalariesBySite(siteId);
        return result.Success ? Ok(result) : NotFound(result);
    }
    
    /// <summary>
    /// Récupère les salariés appartenant à un service spécifique
    /// </summary>
    /// <param name="serviceId">Identifiant du service</param>
    /// <returns>Liste des salariés du service</returns>
    [HttpGet("service/{serviceId}")]
    public async Task<IActionResult> GetSalariesByService(int serviceId)
    {
        var result = await _salarieService.GetSalariesByService(serviceId);
        return result.Success ? Ok(result) : NotFound(result);
    }
    
}
