using ldap_cesi.DTOs.Inputs.Site;
using ldap_cesi.Services.Interfaces;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;

namespace ldap_cesi.Controllers;

[ApiController]
[Route("/api/sites")]
public class SiteController : ControllerBase
{
    private readonly ISiteService _siteService;

    public SiteController(ISiteService siteService)
    {
        _siteService = siteService;
    }
    
    // GET: api/site
    /// <summary>
    /// Récupère la liste de tous les sites.
    /// </summary>
    /// <returns>Retourne une liste de tous les sites.</returns>
    [HttpGet]
    public async Task<ActionResult> GetSites([FromQuery] int pageNumber = 1, [FromQuery] int pageSize = 10)
    {
        var result = await _siteService.GetAll(pageNumber, pageSize);
        return result.Success ? Ok(result) : BadRequest(result);
    }            
    
    // GET: api/site/{id}
    /// <summary>
    /// Récupère un site spécifique par son identifiant.
    /// </summary>
    /// <param name="id">L'identifiant du site à récupérer.</param>
    /// <returns>Retourne le site correspondant à l'identifiant.</returns>
    /// <response code="200">Le site a été récupéré avec succès.</response>
    /// <response code="400">Une erreur s'est produite lors de la récupération du site.</response>
    /// <response code="404">Le site n'a pas été trouvé.</response>
    [HttpGet("{id}")]
    public async Task<ActionResult> GetSite(int id)
    {
        var result = await _siteService.GetById(id);
        return result.Success ? Ok(result) : BadRequest(result);
    }           
    
    // GET: api/site/{id}
    /// <summary>
    /// Récupère un site et ses salariés spécifique par son identifiant.
    /// </summary>
    /// <param name="id">L'identifiant du site à récupérer.</param>
    /// <returns>Retourne le site et ses salariés</returns>
    [HttpGet("/complete/{id}")]
    public async Task<ActionResult> GetSiteWithClient(int id)
    {
        var result = await _siteService.GetByIdWithRelations(id, s => s.Salaries);
        return result.Success ? Ok(result) : BadRequest(result);
    }        
    
    // POST: api/site
    /// <summary>
    /// Crée un nouveau site.
    /// </summary>
    /// <param name="siteCreateDto">Les données du site à créer.</param>
    /// <returns>Retourne l'identifiant du site créé.</returns>
    [HttpPost]
    [Authorize(Roles = "admin")]
    public async Task<ActionResult> CreateSite([FromBody] SiteCreateDto siteCreateDto)
    {
        var result = await _siteService.Create(siteCreateDto);
        return result.Success ? Ok(result) : BadRequest(result);
    }
    
    // PUT: api/site
    /// <summary>
    /// Met à jour un site existant.
    /// </summary>
    /// <param name="siteUpdateDto">Les données du site à mettre à jour.</param>
    /// <returns>Retourne l'objet ou une erreur.</returns>
    [HttpPut]
    [Authorize(Roles = "admin")]
    public async Task<IActionResult> UpdateSite([FromBody] SiteUpdateDto siteUpdateDto)
    {
        var result = await _siteService.Update(siteUpdateDto);
        return result.Success ? Ok(result) : BadRequest(result);
    }
    
    // DELETE: api/site/{id}
    /// <summary>
    /// Supprime un site par son identifiant.
    /// </summary>
    /// <param name="id">L'identifiant du site à supprimer.</param>
    /// <returns>Retourne l'id, ou l'erreur</returns>
    [HttpDelete("{id}")]
    [Authorize(Roles = "admin")]
    public async Task<IActionResult> DeleteSite(int id)
    {
        var result = await _siteService.DeleteWithEntiteCheck(id);
        return result.Success ? Ok(result) : BadRequest(result);
    }
}