using AutoMapper;
using ldap_cesi.DTOs;
using ldap_cesi.DTOs.Inputs.Salarie;
using ldap_cesi.DTOs.Inputs.Service;
using ldap_cesi.DTOs.Outputs.Salarie;
using ldap_cesi.Entities;
using ldap_cesi.Models;
using ldap_cesi.Repository.Services;
using ldap_cesi.Services.Interfaces;
using ldap_cesi.Validator.Salarie;

namespace ldap_cesi.Services;

public class SalarieService : ServiceBase<Salarie, SalarieOutputDetail, SalarieCreateDto, SalarieUpdateDto>, ISalarieService
{
    private IRepositorySalarie _repositorySalarie;
    private readonly IRepositorySite _repositorySite;
    private readonly IRepositoryService _repositoryService;
    private readonly IMapper _mapper;

    public SalarieService(IRepositorySalarie repositorySalarie, IMapper mapper, IRepositorySite repositorySite, IRepositoryService repositoryService, 
                        ILogger<SalarieService> logger, SalarieCreateValidator salarieCreateValidator, SalarieUpdateValidator salarieUpdateValidator) 
        : base(repositorySalarie, mapper, logger, salarieCreateValidator, salarieUpdateValidator)
    {
        _repositorySalarie = repositorySalarie;
        _repositorySite = repositorySite;
        _repositoryService = repositoryService;
        _mapper = mapper;
    }

    public async Task<IResponseDataModel<List<SalarieOutputDetail>>> GetSalariesBySite(int siteId, int pageNumber = 1, int pageSize = 25)
    {
        var site = await _repositorySite.GetByIdAsync(siteId);
        if (site == null)
        {
            return new ResponseDataModel<List<SalarieOutputDetail>>
            {
                Success = false,
                Message = "Site non trouvé",
                StatusCode = 404
            };
        }

        var salaries = await _repositorySalarie.GetSalariesBySiteAsync(siteId);
        var totalCount = salaries.Count;
        var paginatedSalaries = salaries
            .Skip((pageNumber - 1) * pageSize)
            .Take(pageSize)
            .ToList();

        var salariesDto = _mapper.Map<List<SalarieOutputDetail>>(paginatedSalaries);

        return new ResponseDataModel<List<SalarieOutputDetail>>
        {
            Success = true,
            Data = salariesDto,
            TotalPages = (int)Math.Ceiling((double)totalCount / pageSize),
            TotalCount = totalCount,
            PageNumber = pageNumber,
            PageSize = pageSize,
            StatusCode = 200
        };
    }

    public async Task<IResponseDataModel<List<SalarieOutputDetail>>> GetSalariesByService(int serviceId, int pageNumber = 1, int pageSize = 25)
    {
        var service = await _repositoryService.GetByIdAsync(serviceId);
        if (service == null)
        {
            return new ResponseDataModel<List<SalarieOutputDetail>>
            {
                Success = false,
                Message = "Service non trouvé",
                StatusCode = 404
            };
        }

        var salaries = await _repositorySalarie.GetSalariesByServiceAsync(serviceId);
        var totalCount = salaries.Count;
        var paginatedSalaries = salaries
            .Skip((pageNumber - 1) * pageSize)
            .Take(pageSize)
            .ToList();

        var salariesDto = _mapper.Map<List<SalarieOutputDetail>>(paginatedSalaries);

        return new ResponseDataModel<List<SalarieOutputDetail>>
        {
            Success = true,
            Data = salariesDto, // Les salariés paginés
            TotalPages = (int)Math.Ceiling((double)totalCount / pageSize),
            TotalCount = totalCount,
            PageNumber = pageNumber,
            PageSize = pageSize,
            StatusCode = 200
        };
    }
}